import java.util.*;
import java.io.IOException;
import java.io.File;
import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import com.sun.java.swing.JFrame;
import com.sun.java.swing.ImageIcon;

public class DnDEngine extends Object {
    public static final int ACTION_COPY = 0;
    public static final int ACTION_MOVE = 1;

    public static ImageIcon dropFile;
    public static ImageIcon dropFiles;
    public static ImageIcon noDrop;

    static DnDEngine systemDnDEngine;

    DnDSourceAdapter sourceListener = new DnDSourceAdapter();
    DnDTargetAdapter targetListener = new DnDTargetAdapter();

    Component dndSource, dragFrame;
    Vector dndTargets;
    Cursor dragCursor;
    Image dragImage;
    Transferable dragData;
    int dragAction;
    int dropAction;
    boolean MultipleSelection;
    boolean active;

    Object timerLock = new Object();
    boolean canPaint;
    TimerThread timer;

    static {
        systemDnDEngine = new DnDEngine();
        noDrop = new ImageIcon("images/nodrop.gif");
        dropFile = new ImageIcon("images/drop_file.gif");
        dropFiles = new ImageIcon("images/drop_files.gif");
    }

    public static DnDEngine getSystemDnDEngine() {
        return systemDnDEngine;
    }

    public DnDEngine() {
        dndTargets = new Vector();
        active = false;
        timer = new TimerThread();
//        timer.setPriority(Thread.MIN_PRIORITY);
        timer.start();
        timer.suspend();
    }

    public boolean isActive() {
        return active;
    }

    public void registerTarget(Component target) {
        target.addMouseListener(targetListener);
        target.addMouseMotionListener(targetListener);
        dndTargets.addElement(target);
    }

    public boolean isTarget(Component target) {
        return ((! dndSource.equals(target)) && (dndTargets.contains(target)));
    }

    public DnDTargetListener findTarget(Component source, Point p) {
        Point loc = new Point(p);
        loc.translate(source.getLocationOnScreen().x,
                      source.getLocationOnScreen().y);
        for (int i = 0; i < dndTargets.size(); i ++) {
            Point pnt = new Point(loc);
            Component target = (Component) dndTargets.elementAt(i);
            pnt.translate((-1) * target.getLocationOnScreen().x,
                          (-1) * target.getLocationOnScreen().y);
            if (target.contains(pnt))
                return (DnDTargetListener) dndTargets.elementAt(i);

        }
        return null;
    }

    public Point translateFromSourceToTarget(Component source, Point p,
                                             Component target)
   {
        Point loc = new Point(p);
        loc.translate(source.getLocationOnScreen().x,
                      source.getLocationOnScreen().y);
        loc.translate((-1) * target.getLocationOnScreen().x,
                      (-1) * target.getLocationOnScreen().y);
        return loc;
    }

    public void unregisterTarget(Component target) {
        if (isTarget(target)) {
            target.removeMouseMotionListener(targetListener);
            target.removeMouseListener(targetListener);
            dndTargets.removeElement(target);
       }
    }

    public void startDrag(Component source, int action, Cursor dragCursor,
                          Image dragImage, Point startPoint, Transferable data,
                          Component frame, boolean multiple) {
        dndSource = source;
        dragFrame = frame;
        this.dragCursor = dragCursor;
        this.dragImage = dragImage;
        dragData = data;
        MultipleSelection = multiple;
        dndSource.addMouseListener(sourceListener);
        dndSource.addMouseMotionListener(sourceListener);
//        dragFrame.setCursor(dragCursor);
        dragAction = action;
        dropAction = action;
        active = true;
        Point p = translateFromSourceToTarget(source, startPoint, dragFrame);
        dragFrame.getGraphics().drawImage(dragImage, p.x, p.y, dragFrame);
        timer.resume();
    }

    public void stopSourceDrag(boolean drop, DnDSourceListener source, Point dropPoint) {
        if(active) {
          if (drop) {
            dndSource.removeMouseListener(sourceListener);
            dndSource.removeMouseMotionListener(sourceListener);
//            dragFrame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            DnDTargetListener target = findTarget((Component) source, dropPoint);
            DnDDropEvent dnds = new DnDDropEvent((Component) source,
                                          dropAction, dropPoint, dragData);
            dnds.rejectDrop();
            if (target != null) {
                DnDDropEvent dndt = new DnDDropEvent((Component) target, dropAction, 
                                       translateFromSourceToTarget((Component) source,
                                             dropPoint,(Component) target), dragData);
                if (target.dropAccept(dndt))
                    target.drop(dndt);
                if (dndt.isAccepted())
                    dnds.acceptDrop();
            }
            source.dragDropEnd(dnds);
            active = false;
            timer.suspend();
          }
          else {
//            dragFrame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
          }
        }
    }

    public void stopTargetDrag(boolean drop, DnDTargetListener target, Point dropPoint) {
        if(active) {
          if (drop) {
            dndSource.removeMouseListener(sourceListener);
            dndSource.removeMouseMotionListener(sourceListener);
//            dragFrame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            DnDDropEvent dnde = new DnDDropEvent((Component) target,
                                          dropAction, dropPoint, dragData);
            if (target.dropAccept(dnde))
                target.drop(dnde);
            ((DnDSourceListener) dndSource).dragDropEnd(dnde);
            active = false;
          }
          else {
//            dragFrame.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
          }
        }
    }

    public void continueSourceDrag(Component source, Point dragPoint) {
        if (active) {
            DnDDragEvent dnds = new DnDDragEvent(source, dragAction, dragPoint);
            dnds.rejectDrop();
            DnDTargetListener target = findTarget(source, dragPoint);
            if (target != null) {
                DnDDragEvent dndt = new DnDDragEvent((Component) target, dragAction,
                   translateFromSourceToTarget(source, dragPoint,(Component) target));
                target.dragOver(dndt);
                if (dndt.isAccepted())
                    dnds.acceptDrop();
                else
                    dnds.rejectDrop();
            }
            ((DnDSourceListener) source).dragOver(dnds);
            synchronized(timerLock) {
              if (canPaint == true) {
                dragFrame.repaint();
                Point p = translateFromSourceToTarget(source, dragPoint, dragFrame);
                if (dnds.isAccepted()) {
                    dragFrame.getGraphics().drawImage(
                       (MultipleSelection) ? dropFiles.getImage() :
                                             dropFile.getImage(), p.x, p.y, dragFrame);
                }
                else {
                    dragFrame.getGraphics().drawImage(noDrop.getImage(), p.x, p.y,
                                                      dragFrame);
                }
              }
            }
        }
    }

    public void continueTargetDrag(Component target, Point dragPoint) {
        if (active) {
            ((DnDTargetListener) target).dragOver(new DnDDragEvent(target,
                                          dragAction, dragPoint));
//            dragFrame.setCursor(new Cursor(Cursor.HAND_CURSOR));
        }
    }

    public class DnDSourceAdapter extends MouseAdapter implements MouseMotionListener {

      public void mouseMoved(MouseEvent e) {}

      public void mouseDragged(MouseEvent e) {
        if (active) {
          continueSourceDrag(e.getComponent(), new Point(e.getX(), e.getY()));
        }
      }

      public void mouseClicked(MouseEvent e) {}
      public void mousePressed(MouseEvent e) {}
      public void mouseReleased(MouseEvent e) {
        if (active) {
            stopSourceDrag(true,(DnDSourceListener) e.getComponent(), 
                           new Point(e.getX(), e.getY()));
        }
      }

      public void mouseEntered(MouseEvent e) {
        if (active && ((e.getModifiers() & MouseEvent.BUTTON1_MASK) == 1)) {
            continueSourceDrag(e.getComponent(), new Point(e.getX(), e.getY()));
        }
      }

      public void mouseExited(MouseEvent e) {
        stopSourceDrag(false,(DnDSourceListener) e.getComponent(), 
                       new Point(e.getX(), e.getY()));
      }

    }
                                                 
    public class DnDTargetAdapter extends MouseAdapter implements MouseMotionListener {
      public void mouseMoved(MouseEvent e) {
      }

      public void mouseDragged(MouseEvent e) {
        if (active) {
            continueTargetDrag(e.getComponent(), new Point(e.getX(), e.getY()));
        }
      }

      public void mouseClicked(MouseEvent e) {}
      public void mousePressed(MouseEvent e) {}
      public void mouseReleased(MouseEvent e) {
        if (active) {
            stopTargetDrag(true,(DnDTargetListener) e.getComponent(), 
                           new Point(e.getX(), e.getY()));
        }
      }

      public void mouseEntered(MouseEvent e) {
        if (active && ((e.getModifiers() & MouseEvent.BUTTON1_MASK) == 1)) {
            continueTargetDrag(e.getComponent(), new Point(e.getX(), e.getY()));
        }
      }

      public void mouseExited(MouseEvent e) {
          if (active)
              stopTargetDrag(false,(DnDTargetListener) e.getComponent(), 
                             new Point(e.getX(), e.getY()));
      }

    }

    class TimerThread extends Thread {

      public TimerThread()
      {
        canPaint = true;
      }

      public void run() {
        try {
          while(true) {
            sleep(200);
            synchronized(timerLock) {
              if (canPaint == true) {
                canPaint = false;
              }
              else {
                canPaint = true;
              }
              timerLock.notify();
            }
          }
        } 
        catch (InterruptedException  e) {}

      }

    }
}
