/*
 * @(#)JSVoyagerFrame.java	0.1 1999/01/06
 * 
 * Copyright (c) 1999 IS Corp. All Rights Reserved.
 * @author Sergey Izvozchikov
 * 
 */

import java.io.*;
import java.net.URL;
import java.util.*;
import java.lang.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import com.sun.java.swing.preview.*;
import com.sun.java.swing.text.*;
import com.sun.java.swing.undo.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.plaf.metal.*;

public class JSVoyagerFrame extends JFrame {

    private static ResourceBundle resources;

    static {
        try {
            resources = ResourceBundle.getBundle("JSVoyagerFrame", 
                                                 Locale.getDefault());
        } catch (MissingResourceException mre) {
            System.err.println("JSVoyager properties file not found");
            System.exit(1);
        }
        catch (ExceptionInInitializerError e) {
            System.err.println(e.toString());
            System.exit(1);
        }
    }

    private Hashtable commands;
    private Hashtable menuItems;
    private JMenuBar menubar;
    private JToolBar toolbar;
    private JPopupMenu contextMenu;
    private JPanel content;
    private JSVoyagerDesktopPane desktop;
    private JSVoyagerPanel activePane = null;

    static final Integer PANELSLAYER = new Integer(6);
    static final Integer HELPLAYER = new Integer(5);

    private JComponent status;
    private JProgressBar progress;

    protected FileDialog fileDialog;

    public static final String imageSuffix = "Image";
    public static final String labelSuffix = "Label";
    public static final String actionSuffix = "Action";
    public static final String tipSuffix = "Tooltip";

    public JSVoyagerFrame() {
      super(resources.getString("Title"));
      setBackground(UIManager.getColor("control"));
      final int inset = 50;
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      setBounds ( inset, inset, screenSize.width - inset*2, screenSize.height - inset*2 );
	commands = new Hashtable();
	Action[] actions = getActions();
	for (int i = 0; i < actions.length; i++) {
	    Action a = actions[i];
	    commands.put(a.getValue(Action.NAME), a);
	}
	
	menuItems = new Hashtable();
	setJMenuBar(createMenubar());	
      buildContent();
    }

    protected JMenuBar createMenubar() {
	JMenuItem mi;
	JMenuBar mb = new JMenuBar();

	mb.setOpaque(true);
	String[] menuKeys = tokenize(getResourceString("menubar"));
	for (int i = 0; i < menuKeys.length; i++) {
	    JMenu m = createMenu(menuKeys[i]);
	    if (m != null) {
		mb.add(m);
	    }
	}
	return mb;
    }

    protected JMenu createMenu(String key) {
	String[] itemKeys = tokenize(getResourceString(key));
	JMenu menu = new JMenu(getResourceString(key + "Label"));
	for (int i = 0; i < itemKeys.length; i++) {
	    if (itemKeys[i].equals("-")) {
		menu.addSeparator();
	    } else {
		JMenuItem mi = createMenuItem(itemKeys[i]);
	        menuItems.put(itemKeys[i], mi);
		menu.add(mi);
	    }
	}
	return menu;
    }

    protected JMenuItem createMenuItem(String cmd) {
	JMenuItem mi = new JMenuItem(getResourceString(cmd + labelSuffix));
      URL url = getResource(cmd + imageSuffix);
	if (url != null) {
	    mi.setHorizontalTextPosition(JButton.RIGHT);
	    mi.setIcon(new ImageIcon(url));
	}
	String astr = getResourceString(cmd + actionSuffix);
	if (astr == null) {
	    astr = cmd;
	}
	mi.setActionCommand(astr);
	Action a = getAction(astr);
	if (a != null) {
	    mi.addActionListener(a);
	    a.addPropertyChangeListener(createActionChangeListener(mi));
	    mi.setEnabled(a.isEnabled());
	} else {
	    mi.setEnabled(false);
	}
	return mi;
    }

    protected JMenuItem getMenuItem(String cmd) {
	return (JMenuItem) menuItems.get(cmd);
    }

    protected Action getAction(String cmd) {
	return (Action) commands.get(cmd);
    }

    protected String getResourceString(String nm) {
	String str;
	try {
	    str = resources.getString(nm);
	} catch (MissingResourceException mre) {
	    str = null;
	}
	return str;
    }

    protected URL getResource(String key) {
	String name = getResourceString(key);
	if (name != null) {
	    URL url = this.getClass().getResource(name);
	    return url;
	}
	return null;
    }

    protected Container getToolbar() {
	return toolbar;
    }

    protected JMenuBar getMenubar() {
	return menubar;
    }

    public Action[] getActions() {
        return defaultActions;
    }

    private Component createToolbar() {
	toolbar = new JToolBar();
	String[] toolKeys = tokenize(getResourceString("toolbar"));
	for (int i = 0; i < toolKeys.length; i++) {
	    if (toolKeys[i].equals("-")) {
		toolbar.add(Box.createHorizontalStrut(5));
	    } else {
		toolbar.add(createTool(toolKeys[i]));
	    }
	}
	toolbar.add(Box.createHorizontalGlue());
	return toolbar;
    }

    protected Component createTool(String key) {
	return createToolbarButton(key);
    }

    protected JButton createToolbarButton(String key) {
	URL url = getResource(key + imageSuffix);
        JButton b = new JButton(new ImageIcon(url)) {
            public float getAlignmentY() { return 0.5f; }
	};
      b.setRequestFocusEnabled(false);
      b.setMargin(new Insets(1,1,1,1));

	String astr = getResourceString(key + actionSuffix);
	if (astr == null) {
	    astr = key;
	}
	Action a = getAction(astr);
	if (a != null) {
	    b.setActionCommand(astr);
	    b.addActionListener(a);
	} else {
	    b.setEnabled(false);
	}

	String tip = getResourceString(key + tipSuffix);
	if (tip != null) {
	    b.setToolTipText(tip);
	}
 
      return b;
    }

    private JPopupMenu createContextMenu() {
	JPopupMenu pm = new JPopupMenu();
	String[] toolKeys = tokenize(getResourceString("popup"));
	for (int i = 0; i < toolKeys.length; i++) {
	    if (toolKeys[i].equals("-")) {
		pm.addSeparator();
	    } else {
		pm.add(createMenuItem(toolKeys[i]));
		/*
                Action a = createPopup(toolKeys[i]);
	        if (a != null)
                    pm.add(a);
                */
	    }
	}
	return pm;
    }

    protected Action createPopup(String key) {
	String astr = getResourceString(key + actionSuffix);
	if (astr == null) {
	    astr = key;
	}
	Action a = getAction(astr);
        return a;
    }

    protected String[] tokenize(String input) {
	Vector v = new Vector();
	StringTokenizer t = new StringTokenizer(input);
	String cmd[];

	while (t.hasMoreTokens())
	    v.addElement(t.nextToken());
	cmd = new String[v.size()];
	for (int i = 0; i < cmd.length; i++)
	    cmd[i] = (String) v.elementAt(i);

	return cmd;
    }

    protected PropertyChangeListener createActionChangeListener(JMenuItem b) {
	return new ActionChangedListener(b);
    }

    private class ActionChangedListener implements PropertyChangeListener {
        JMenuItem menuItem;
        
        ActionChangedListener(JMenuItem mi) {
            super();
            this.menuItem = mi;
        }
        public void propertyChange(PropertyChangeEvent e) {
            String propertyName = e.getPropertyName();
            if (e.getPropertyName().equals(Action.NAME)) {
                String text = (String) e.getNewValue();
                menuItem.setText(text);
            } else if (propertyName.equals("enabled")) {
                Boolean enabledState = (Boolean) e.getNewValue();
                menuItem.setEnabled(enabledState.booleanValue());
            }
        }
    }

    protected void buildContent() {
        desktop = new JSVoyagerDesktopPane();
        JScrollPane desktopPane = new JScrollPane(desktop,
                               JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                               JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        content = new JPanel(new BorderLayout());
        content.add("North", createToolbar());
//        content.add("Center", desktopPane);
        content.add("Center", desktop);
        getContentPane().add(content);
        contextMenu = createContextMenu();
        newPanel();
    }

    // --- global methods -----------------------------------
/*
    public static void setDefaulCursor() {
        setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    public static void setWaitCursor() {
        setCursor(new Cursor(Cursor.WAIT_CURSOR));
    }
*/

    // --- action implementations -----------------------------------
    private UndoAction undoAction = new UndoAction();

    private Action[] defaultActions = {
      new UpAction(),
      new NewAction(),
      new CreateAction(),
      new FindAction(),
      new RenameAction(),
      new PropertiesAction(),
      new ExitAction(),
      new CutAction(),
      new CopyAction(),
      new PasteAction(),
      new DeleteAction(),
      undoAction,
      new SelectAllAction(),
      new ViewListAction(),
      new ViewDetailsAction(),
      new TileAction(),
      new CloseAction(),
      new RefreshAction(),
      new AboutAction(),
      new HelpAction()
    };

    class UndoAction extends AbstractAction {
	public UndoAction() {
	    super("Undo");
	    setEnabled(false);
	}
	public void actionPerformed(ActionEvent e) {
/*
	    try {
		if (activePane != null)
              activePane.getUndoManager().undo();
	    } catch (CannotUndoException ex) {
		System.out.println("Unable to undo: " + ex);
		ex.printStackTrace();
	    }
	    update();
	    redoAction.update();
*/
	}

/*
	protected void update() {
	    if (activePane != null && activePane.getUndoManager().canUndo()) {
		setEnabled(true);
		putValue(Action.NAME, activePane.getUndoManager().getUndoPresentationName());
	    }
	    else {
		setEnabled(false);
		putValue(Action.NAME, "Undo");
	    }
	}
*/
    }

    class UpAction extends AbstractAction {

	UpAction() {
	    super("up");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              activePane.upToParent();
          }
	}
    }

    class NewAction extends AbstractAction {

	NewAction() {
	    super("new");
	}

      public void actionPerformed(ActionEvent e) {
        newPanel();
	}
    }

    class CreateAction extends AbstractAction {

	CreateAction () {
	    super("Create");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              String folder = JOptionPane.showInputDialog("Name for new folder : ");
              if (folder != null) {
                  activePane.createFolder(folder);
              }
          }
	}
    }

    class SelectAllAction extends AbstractAction {

	SelectAllAction() {
	    super("SelectAll");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              activePane.selectAll();
          }
	}
    }

    class RefreshAction extends AbstractAction {

	RefreshAction() {
	    super("Refresh");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              activePane.refreshAll();
          }
	}
    }

    class ExitAction extends AbstractAction {

	ExitAction() {
	    super("exit");
	}

      public void actionPerformed(ActionEvent e) {
	    System.exit(0);
	}
    }

    class FindAction extends AbstractAction {

	FindAction() {
	    super("Find");
	}

      public void actionPerformed(ActionEvent e) {
          String template = JOptionPane.showInputDialog("Find file(mask) :");
          if (activePane != null) {
              findFile(activePane.getCurrentPath(), template);
          }
	}
    }

    class PropertiesAction extends AbstractAction {

	PropertiesAction() {
	    super("Properties");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              activePane.viewProperties();
          }
	}
    }

    class RenameAction extends AbstractAction {

	RenameAction() {
	    super("Rename");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              File file = activePane.getFile();
              if (file != null) {
                String newname = JOptionPane.showInputDialog("Rename " + 
                                 file.getName() + " to :");
                if (newname != null) {
                  File newFile = new File(file.getParent(), newname);
                  file.renameTo(newFile);
                  activePane.refreshAll();
                }
              }
              else
                  JOptionPane.showMessageDialog(JSVoyagerFrame.this,
                          "No file is selected !");
          }
	}
    }

    class CutAction extends AbstractAction {

	CutAction() {
	    super("Cut");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              activePane.cutFiles();
          }
	}
    }

    class CopyAction extends AbstractAction {

	CopyAction() {
	    super("Copy");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              activePane.copyFiles();
          }
	}
    }

    class PasteAction extends AbstractAction {

	PasteAction() {
	    super("Paste");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              boolean success = activePane.pasteFiles();
/*
              setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
              if (success)
		  JOptionPane.showMessageDialog(JSVoyagerFrame.this, 
		                                "Files have been deleted !");
              else
		  JOptionPane.showMessageDialog(JSVoyagerFrame.this, 
		                                "Files can't be deleted !");
              activePane.refresh();
*/
          }
	}
    }

    class DeleteAction extends AbstractAction {

	DeleteAction() {
	    super("Delete");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
//              setCursor(new Cursor(Cursor.WAIT_CURSOR));
              boolean success = activePane.deleteFiles();
/*
              setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
              if (success)
		  JOptionPane.showMessageDialog(JSVoyagerFrame.this, 
		                                "Files have been deleted !");
              else
		  JOptionPane.showMessageDialog(JSVoyagerFrame.this, 
		                                "Files can't be deleted !");
              activePane.refresh();
*/
          }
	}
    }

    class ViewListAction extends AbstractAction {

	ViewListAction() {
	    super("List");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              activePane.getListPane().showList();
          }
	}
    }

    class ViewDetailsAction extends AbstractAction {

	ViewDetailsAction() {
	    super("Details");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
              activePane.getListPane().showTable();
          }
	}
    }

    class TileAction extends AbstractAction {

	TileAction() {
	    super("Tile");
	}

      public void actionPerformed(ActionEvent e) {
	}
    }

    class CloseAction extends AbstractAction {

	CloseAction() {
	    super("Close");
	}

      public void actionPerformed(ActionEvent e) {
          if (activePane != null) {
//              activePane.close();
          }
	}
    }

    class AboutAction extends AbstractAction {

	AboutAction() {
	    super("About");
	}

      public void actionPerformed(ActionEvent e) {
          JOptionPane.showMessageDialog(JSVoyagerFrame.this, 
          "Voyager (Swing Edition)\n Copyright(C) 1999 Sergey Izvozchikov");
	}
    }

    class HelpAction extends AbstractAction {

	HelpAction() {
	    super("Help");
	}

      public void actionPerformed(ActionEvent e) {
          openHelpWindow();
	}
    }

    public void openHelpWindow() {
	JSVoyagerHelp help = new JSVoyagerHelp();
	desktop.add((JInternalFrame) help, HELPLAYER);
	try { 
          help.setSelected(true); 
	} catch (java.beans.PropertyVetoException e2) {}
    }

    public void newPanel() {
	JSVoyagerPanel newpane = new JSVoyagerPanel(
                      new ImageIcon(getResource("folders" + imageSuffix)),
                      new ImageIcon(getResource("files" + imageSuffix)));
	desktop.add((JInternalFrame) newpane, PANELSLAYER);
	try { 
            newpane.setContextMenu(contextMenu);
	    newpane.setSelected(true); 
	} catch (java.beans.PropertyVetoException e2) {}
    }

    public void findFile(File root, String template) {
        if (root == null)
            return;
        JOptionPane.showMessageDialog(JSVoyagerFrame.this, 
                    "This function is not implemented yet !");
    }

    class JSVoyagerDesktopPane extends JDesktopPane
                          implements UndoableEditListener
    {
        public JSVoyagerDesktopPane() {
            setDesktopManager(new JSVoyagerDesktopManager());
        }

        public void undoableEditHappened(UndoableEditEvent e) {
//            undoAction.update();
        }
    }

    class JSVoyagerDesktopManager extends DefaultDesktopManager
    {
        public void activateFrame(JInternalFrame f) {
            super.activateFrame(f);
            activePane = (JSVoyagerPanel) f;
        }

        public void closeFrame(JInternalFrame f) {
            super.closeFrame(f);
            JInternalFrame[] frames = desktop.getAllFramesInLayer(PANELSLAYER.intValue());
            if (frames.length == 0)
                activePane = null;
        }
    }

}
