/*
 * @(#)JSVoyagerHelp.java	0.1 1999/01/06
 * 
 * Copyright (c) 1999 IS Corp. All Rights Reserved.
 * @author Sergey Izvozchikov
 * 
 */

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import java.net.URL;
import java.net.MalformedURLException;
import java.io.*;
import com.sun.java.swing.*;
import com.sun.java.swing.text.*;
import com.sun.java.swing.event.*;

public class JSVoyagerHelp extends JInternalFrame {
    Vector history;
    int historyIndex;
    JTextField anchor;
    HtmlPane html;
  
    public JSVoyagerHelp() {
	super("Help", true, true, true, true);
      history = new Vector();
      historyIndex = 0;
      anchor = new JTextField();
      anchor.setEditable(false);
	setIconifiable(true);
     	setFrameIcon((Icon) UIManager.get("Tree.openIcon"));
	setBounds( 200, 25, 400, 400);
	JPanel p = new JPanel();

     	p.setOpaque(true);
     	p.setBackground(UIManager.getColor("window"));
	p.setLayout(new BorderLayout() );
	html = new HtmlPane(this);

	p.add(html, BorderLayout.CENTER);
	p.add(buildToolBar(), BorderLayout.NORTH);

	getContentPane().add(p);
    }

    protected JToolBar buildToolBar() {
        JToolBar bar = new JToolBar();
	  bar.setFloatable(false);
        JButton back = (JButton) bar.add(new JButton(new ImageIcon("images/back.gif")));
        back.addActionListener(new BackAction());
        JButton forward = (JButton) bar.add(new JButton(new ImageIcon("images/forward.gif")));
        forward.addActionListener(new ForwardAction());
        bar.addSeparator();
        bar.add(new JLabel("Location :"));
        bar.add(anchor);
        return bar;
    }

    public void pack() {
       Dimension size = getPreferredSize();
       setSize(size.width, size.height);
    }

    class BackAction extends AbstractAction {

	BackAction() {
          super("back");
	}

      public void actionPerformed(ActionEvent e) {
          if (historyIndex > 0) {
              historyIndex--;
              html.linkActivated((URL) history.elementAt(historyIndex), false);
          }
	}
    }

    class ForwardAction extends AbstractAction {

	ForwardAction() {
          super("forward");
	}

      public void actionPerformed(ActionEvent e) {
          if (historyIndex < history.size() - 1) {
              historyIndex++;
              html.linkActivated((URL) history.elementAt(historyIndex), false);
          }
	}
    }
}

class HtmlPane extends JPanel implements HyperlinkListener {
    JEditorPane html;
    JSVoyagerHelp parent;

    public HtmlPane(JSVoyagerHelp parent) {
      this.parent = parent;
      setLayout(new BorderLayout());
	try {
	    File f = new File ("HelpFiles/toc.html");
	    String s = f.getAbsolutePath();
	    s = "file:"+s;
	    URL url = new URL(s);
          parent.history.addElement(url);
          parent.anchor.setText(url.toString());
	    html = new JEditorPane(s);
	    html.setEditable(false);
	    html.addHyperlinkListener(this);

	    JScrollPane scroller = new JScrollPane();
	    JViewport vp = scroller.getViewport();
	    vp.add(html);
	    add(scroller, BorderLayout.CENTER);
	} catch (MalformedURLException e) {
	    System.out.println("Malformed URL: " + e);
	} catch (IOException e) {
	    System.out.println("IOException: " + e);
	}	
    }

    /**
     * Notification of a change relative to a 
     * hyperlink.
     */
    public void hyperlinkUpdate(HyperlinkEvent e) {
	if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
            System.out.println("Link has activated" + e.getURL());
	    linkActivated(e.getURL(), true);
	}
    }

    /**
     * Follows the reference in an
     * link.  The given url is the requested reference.
     * By default this calls <a href="#setPage">setPage</a>,
     * and if an exception is thrown the original previous
     * document is restored and a beep sounded.  If an 
     * attempt was made to follow a link, but it represented
     * a malformed url, this method will be called with a
     * null argument.
     *
     * @param u the URL to follow
     */
    protected void linkActivated(URL u, boolean toIndex) {
	Cursor c = html.getCursor();
	Cursor waitCursor = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
	html.setCursor(waitCursor);
	SwingUtilities.invokeLater(new PageLoader(u, c, toIndex));
    }

    /**
     * temporary class that loads synchronously (although
     * later than the request so that a cursor change
     * can be done).
     */
    class PageLoader implements Runnable {
	
	PageLoader(URL u, Cursor c, boolean toIndex) {
	    url = u;
	    cursor = c;
          this.toIndex = toIndex;
	}

        public void run() {
	    if (url == null) {
		// restore the original cursor
		html.setCursor(cursor);

		// PENDING(prinz) remove this hack when 
		// automatic validation is activated.
		Container parent = html.getParent();
		parent.repaint();
	    } else {
		Document doc = html.getDocument();
		try {
		    html.setPage(url);
                    parent.anchor.setText(url.toString());
                    if (toIndex) {
                        parent.history.addElement(url);
                        parent.historyIndex = parent.history.size() - 1;
                    }
		} catch (IOException ioe) {
		    html.setDocument(doc);
		    getToolkit().beep();
		} finally {
		    // schedule the cursor to revert after
		    // the paint has happended.
		    url = null;
		    SwingUtilities.invokeLater(this);
		}
	    }
	}

	URL url;
	Cursor cursor;
	boolean toIndex;
    }

}
