/*
 * @(#)JSVoyagerList.java	0.1 1999/01/06
 * 
 * Copyright (c) 1999 IS Corp. All Rights Reserved.
 * @author Sergey Izvozchikov
 * 
 */

import com.sun.java.swing.*;
import com.sun.java.swing.JOptionPane;
import com.sun.java.swing.event.*;
import com.sun.java.swing.table.*;
import com.sun.java.swing.plaf.basic.BasicListCellRenderer;

import java.util.*;
import java.io.IOException;
import java.io.File;
import java.awt.*;
import java.awt.event.*;
//import java.awt.dnd.*;

public class JSVoyagerList extends JPanel implements DnDSourceListener,
                                          MouseMotionListener, MouseListener
{
    Vector mouseListeners = new Vector();
    Vector mouseMotionListeners = new Vector();

    public static final int LIST_MODE = 1;
    public static final int TABLE_MODE = 2;

    public static ImageIcon foldersIcon;
    public static ImageIcon filesIcon;

    public static Cursor dragCursor;
    public static Cursor dropCursor;
    public static Cursor noDropCursor;

    JSVoyagerFileSelection filesSelection;
    File currentPath;

    JSVList list; 
    JTable table;
    CardLayout layout;
    int viewMode;
    JSVoyagerListModel dataModel;
    String newline = "\n";
    String nullString = ""; 
    String folderString = "Directory"; 
    ListSelectionModel listSelectionModel;
    boolean isPopupShowed = false;

    public JSVoyagerList(File path, ImageIcon foldersIcon, ImageIcon filesIcon)
    {
        this.foldersIcon = foldersIcon;
        this.filesIcon = filesIcon;
//        dragCursor = getToolkit().createCustomCursor(this.filesIcon.getImage(),
//                                                     new Point(0, 0), "dragCursor");

        //The data 
        String[] columnNames = { "Name", "Length", "Modification Date" };
        // Types of the columns.
        Class[] columnClasses = { String.class, String.class, Date.class};


        // Transferable object with information about selected files
        filesSelection = new JSVoyagerFileSelection();

        //Build the model.
        dataModel = new JSVoyagerListModel(columnNames, columnClasses);
//        loadDirectory(path);
        dataModel.setPath(path);
        currentPath = path;

        list = new JSVList(dataModel);
        list.addMouseListener(this);
        list.addMouseMotionListener(this);
        list.addDragMotionListener(this);
        list.setCellRenderer(new BasicListCellRenderer() {
            public Component getListCellRendererComponent(JList l, 
                                                          Object value,
                                                          int i,
                                                          boolean s,
                                                          boolean f) {
                File file = new File(currentPath,(String) value);
                if (file.isDirectory())
                  setIcon(JSVoyagerList.foldersIcon);
                else
                  setIcon(JSVoyagerList.filesIcon);
                return super.getListCellRendererComponent(l,
                                                          value,
                                                          i, s, f);
            }
        });

        listSelectionModel = list.getSelectionModel();
        listSelectionModel.addListSelectionListener(
                                new SharedListSelectionHandler());
        listSelectionModel.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        JScrollPane listPane = new JScrollPane(list,
                               JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                               JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        table = new JTable(dataModel);
        table.addMouseListener(this);
        table.addMouseMotionListener(this);
        table.setSelectionModel(listSelectionModel);
        table.setShowGrid(false);
        table.getTableHeader().setReorderingAllowed(true);
        JScrollPane tablePane = JTable.createScrollPaneForTable(table);

/*
                if (newMode.equals("SINGLE_SELECTION")) {
                    listSelectionModel.setSelectionMode(
                        ListSelectionModel.SINGLE_SELECTION);
                } else if (newMode.equals("SINGLE_INTERVAL_SELECTION")) {
                    listSelectionModel.setSelectionMode(
                        ListSelectionModel.SINGLE_INTERVAL_SELECTION);
                } else {
                    listSelectionModel.setSelectionMode(
                        ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
                }
*/

        //do the layout
        layout = new CardLayout();
        setLayout(layout);
        add(listPane, "list");
        add(tablePane, "table");
        listPane.setPreferredSize(new Dimension(300, 100));
        tablePane.setPreferredSize(new Dimension(300, 100));
        layout.show(this, "list");
        viewMode = LIST_MODE;
    }

    public JList getList() {
        return list;
    }

    public JSVoyagerFileSelection getSelection() {
        return filesSelection;
    }

    public String getElementAt(int index) {
        return (String) dataModel.getValueAt(index, 0);
    }

    public void selectAll() {
        ListModel lm = list.getModel();
        list.setSelectionInterval(0, lm.getSize() - 1);
        TableModel tm = table.getModel();
        table.setRowSelectionInterval(0, tm.getRowCount() - 1);
    }

    public void addMouseTracker(MouseListener l) {
        list.addMouseListener(l);
        table.addMouseListener(l);
    }

    public void setWaitCursor() {
        if (viewMode == LIST_MODE) {
            list.setCursor(new Cursor(Cursor.WAIT_CURSOR));
        } 
        else if (viewMode == TABLE_MODE) {
            table.setCursor(new Cursor(Cursor.WAIT_CURSOR));
        }
    }

    public void setDefaultCursor() {
        if (viewMode == LIST_MODE) {
            list.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        } 
        else if (viewMode == TABLE_MODE) {
            table.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
    }

    public void showList() {
        layout.show(this, "list");
        viewMode = LIST_MODE;
    }

    public void showTable() {
        layout.show(this, "table");
        viewMode = TABLE_MODE;
    }

    public void loadDirectory(File path) {
        setWaitCursor();
        if (path != null) {
          dataModel.setPath(path);
          currentPath = path;
          filesSelection.clearSelection();
          list.revalidate();
          table.revalidate();
          list.repaint();
          table.repaint();
          repaint();
        }
        setDefaultCursor();
    }
 
/*
    protected Frame getFrame() {
        for (Container p = getParent(); p != null; p = p.getParent()) {
            if (p instanceof Frame) {
                return (Frame) p;
            }
        }
        return null;
    }
*/

    public synchronized void addMouseListener(MouseListener l) {
        mouseListeners.addElement(l);
    }

    public synchronized void removeMouseListener(MouseListener l) {
        mouseListeners.removeElement(l);
    }

    public synchronized void addMouseMotionListener(MouseMotionListener l) {
        mouseMotionListeners.addElement(l);
    }

    public synchronized void removeMouseMotionListener(MouseMotionListener l) {
        mouseMotionListeners.removeElement(l);
    }

    // MouseListener implementation
    public void mouseClicked(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseListener target = (MouseListener) targets.elementAt(i);
        target.mouseClicked(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

    public void mousePressed(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseListener target = (MouseListener) targets.elementAt(i);
        target.mousePressed(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
      if ( e.isPopupTrigger() )
        isPopupShowed = true;
    }

    public void mouseReleased(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseListener target = (MouseListener) targets.elementAt(i);
        target.mouseReleased(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
      //if ( e.isPopupTrigger() )
        isPopupShowed = false;
    }

    public void mouseEntered(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseListener target = (MouseListener) targets.elementAt(i);
        target.mouseEntered(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

    public void mouseExited(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseListener target = (MouseListener) targets.elementAt(i);
        target.mouseExited(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

    // MouseMotionListener implementation
    public void mouseMoved(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseMotionListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseMotionListener target = (MouseMotionListener) targets.elementAt(i);
        target.mouseMoved(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

    public void mouseDragged(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseMotionListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseMotionListener target = (MouseMotionListener) targets.elementAt(i);
        target.mouseDragged(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
      if (isPopupShowed == false)
        startDnDOperation( new Point(e.getX(), e.getY()) );
    }

    public void startDnDOperation(Point startPoint) {
        DnDEngine dndEngine = DnDEngine.getSystemDnDEngine();
        if (viewMode == LIST_MODE && ! dndEngine.isActive()) {
            dndEngine.startDrag(this, DnDEngine.ACTION_COPY, new Cursor(Cursor.MOVE_CURSOR),
                                filesIcon.getImage(), startPoint,
                                new JSVoyagerFileSelection(filesSelection),
                                JOptionPane.getRootFrame(), 
                                filesSelection.isMultipleSelection());
//            list.setCursor(dragCursor);
            list.setCursor(new Cursor(Cursor.MOVE_CURSOR));
            list.setDragMode(true);
        }
    }

    // DnDSourceListener implementation
    public void dragOver(DnDDragEvent de) {
        if (de.isAccepted()) {
            if (list.getCursor().getType() != Cursor.HAND_CURSOR)
                list.setCursor(new Cursor(Cursor.HAND_CURSOR));
//            list.setCursor(dropCursor);
        }
        else {
            if (list.getCursor().getType() != Cursor.MOVE_CURSOR)
                list.setCursor(new Cursor(Cursor.MOVE_CURSOR));
//            list.setCursor(noDropCursor);
        }
    }

    public void dragDropEnd(DnDDropEvent de) {
        list.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        list.setDragMode(false);
    }

    class SharedListSelectionHandler implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) { 
            ListSelectionModel lsm = (ListSelectionModel) e.getSource();
            if (! lsm.isSelectionEmpty()) {
		filesSelection.clearSelection();
                // Find out which indexes are selected.
                int minIndex = lsm.getMinSelectionIndex();
                int maxIndex = lsm.getMaxSelectionIndex();
                for (int i = minIndex; i <= maxIndex; i++) {
                    if (lsm.isSelectedIndex(i)) {
                        filesSelection.addFile(new File(currentPath, getElementAt(i)));
                    }
                }
            }
        }
    }

    class JSVList extends JList
    {
        Vector dragMotionListeners;
        boolean dragMode = false;

        public JSVList(JSVoyagerListModel datamodel) {
            super(datamodel);
        }

        public void setDragMode(boolean mode) {
            dragMode = mode;
        }

        protected void processMouseMotionEvent(MouseEvent e) {
            if (! dragMode) {
                super.processMouseMotionEvent(e);
            }
            else {
              if (e.getID() == MouseEvent.MOUSE_MOVED) {
                  mouseMoved(e);
              }
              else if (e.getID() == MouseEvent.MOUSE_DRAGGED) {
                  mouseDragged(e);
              }
            }
        }

        public synchronized void addDragMotionListener(MouseMotionListener l) {
            if (dragMotionListeners == null)
                dragMotionListeners = new Vector();
            dragMotionListeners.addElement(l);
        }

        public synchronized void removeDragMotionListener(MouseMotionListener l) {
            if (dragMotionListeners != null)
                dragMotionListeners.removeElement(l);
        }

        public void mouseMoved(MouseEvent e) {
          Vector targets;
          synchronized (this) {
            targets = (Vector) dragMotionListeners.clone();
          }
          for (int i = 0; i < targets.size(); i++) {
            MouseMotionListener target = (MouseMotionListener) targets.elementAt(i);
            target.mouseMoved(e);
          }
        }

        public void mouseDragged(MouseEvent e) {
          Vector targets;
          synchronized (this) {
            targets = (Vector) dragMotionListeners.clone();
          }
          for (int i = 0; i < targets.size(); i++) {
            MouseMotionListener target = (MouseMotionListener) targets.elementAt(i);
            target.mouseDragged(e);
          }
        }
    }

    class JSVoyagerListModel extends AbstractListModel
                          implements TableModel {

        public String[] columnNames;
        public Class[] columnClasses;
        public String[] Files;
        File path;

        public JSVoyagerListModel(String[] columnNames,
                                  Class[] columnClasses) {
            super();
            this.columnNames = columnNames;
            this.columnClasses = columnClasses;
            Files = null;
        }

        public void setPath(File path) {
            this.path = path;
            Files = path.list();
            if (Files != null) 
                super.fireContentsChanged(this, 0, Files.length - 1);
        }

        private TableModel tableModel = new AbstractTableModel() {
            public String getColumnName(int column) {
                return columnNames[column];
            }
/*
            public Class getColumnClass(int column) {
                return columnClasses[column];
            }
*/
            public int getRowCount() { 
                if (Files != null)
                  return Files.length;
                else
                  return 0;
            }
            public int getColumnCount() {
                return columnNames.length;
            }
            public Object getValueAt(int row, int column) {
              if (Files != null) {
                File file = new File(path, Files[row]); 
                if (file.isDirectory()) { 
                  switch(column) {
                    case 0: return file.getName();
                    case 1: return folderString;
                    case 2: return nullString;
                  }
                } else {
                  switch(column) {
                    case 0: return file.getName();
                    case 1: Long len = new Long(file.length()); return len.toString();
                    case 2: Date d = new Date(file.lastModified()); return d.toString();
                  }
                }
              }
              return null;
            }
            public boolean isCellEditable(int row, int column) {
//                if (column == 0) return true;
                return false;
            }
            public void setValueAt(Object value, int row, int column) {
/*
                String newValue = (String)value;
                String[] rowData = (String [])elementAt(row);
                rowData[column] = newValue;
*/
                list.revalidate(); 
            }
        };

        public int getSize() {
            if (Files != null) 
                return Files.length;
            else
                return 0;
        }

        public Object getElementAt(int i) {
            if (Files != null)
                return Files[i];
            else
                return null;
        }

        //Implement the TableModel interface.
        public int getRowCount() {
            return tableModel.getRowCount();
        }
        public int getColumnCount() {
            return tableModel.getColumnCount();
        }
        public String getColumnName(int columnIndex) {
            return tableModel.getColumnName(columnIndex);
        }
        public Class getColumnClass(int columnIndex) {
            return tableModel.getColumnClass(columnIndex);
        }
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return tableModel.isCellEditable(rowIndex, columnIndex);
        }
        public Object getValueAt(int rowIndex, int columnIndex) {
            return tableModel.getValueAt(rowIndex, columnIndex);
        }
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            tableModel.setValueAt(aValue, rowIndex, columnIndex);
        }
        public void addTableModelListener(TableModelListener l) {
            tableModel.addTableModelListener(l);
        }
        public void removeTableModelListener(TableModelListener l) {
            tableModel.removeTableModelListener(l);
        }

    }

class DefaultListCellRenderer extends JLabel implements ListCellRenderer {
    public DefaultListCellRenderer() {
        setOpaque(true);
    }

    public Component getListCellRendererComponent(
        JList list, 
        Object value, 
        int index, 
        boolean isSelected, 
        boolean cellHasFocus) 
    {
        setText((String) value);
        setBackground(isSelected ? Color.red : Color.white);
        setForeground(isSelected ? Color.white : Color.black);
        return this;
    }
}
}
