/*
 * @(#)JSVoyagerPanel.java	0.1 1999/01/06
 * 
 * Copyright (c) 1999 IS Corp. All Rights Reserved.
 * @author Sergey Izvozchikov
 * 
 */

import com.sun.java.swing.event.TreeSelectionListener;
import com.sun.java.swing.event.TreeSelectionEvent;
import com.sun.java.swing.tree.TreeSelectionModel;
import com.sun.java.swing.tree.TreePath;
import com.sun.java.swing.plaf.basic.BasicTreeCellRenderer;
import com.sun.java.swing.ImageIcon;
import com.sun.java.swing.*;
import java.util.Date;
import java.net.URL;
import java.net.URLConnection;
import java.net.MalformedURLException;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import com.sun.java.swing.JScrollPane;
import com.sun.java.swing.JSplitPane;
import com.sun.java.swing.JInternalFrame;
import com.sun.java.swing.ProgressMonitor;
import com.sun.java.swing.JProgressBar;
import java.awt.*;
import java.awt.event.*;

public class JSVoyagerPanel extends JInternalFrame {

    static String[] permissions = {"read", "write", "execute", "delete"}; 

    static int openFrameCount = 0;
    static final int offset = 30;

    private JSVoyagerTree treePane;
    private JSVoyagerList listPane;
    private JPopupMenu contextMenu;
    private JSVoyagerViewer viewerPane;
    private JSplitPane splitView;
    private ImageIcon foldersIcon, filesIcon;

    TreePath currentTreePath;
    File currentFilePath;

    JSVoyagerFileSelection filesClipboard;
    boolean moveFlag = false;

    boolean progressStop;
    Object progressLock = new Object();

    boolean operationSuccess;
    Object operationLock = new Object();

    public JSVoyagerPanel(ImageIcon foldersIcon, ImageIcon filesIcon) {
        super("", true, true, true, true);
        setIconifiable(true);
        setFrameIcon((Icon) UIManager.get("Tree.openIcon"));
        setBounds( 200, 25, 400, 400);
        this.foldersIcon = foldersIcon;
        this.filesIcon = filesIcon;

        currentFilePath = new File(File.separator);
        setTitle(currentFilePath.getAbsolutePath());
        listPane = new JSVoyagerList(currentFilePath, foldersIcon, filesIcon);
        treePane = new JSVoyagerTree(this, currentFilePath);
        viewerPane = new JSVoyagerViewer();
        currentTreePath = new TreePath(treePane.getRoot());

        //Listen for when the selection changes.
        treePane.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                JSVoyagerTreeNode node = (JSVoyagerTreeNode)
                                   (e.getPath().getLastPathComponent());
                currentTreePath = e.getPath();
                currentFilePath = node.getFile();
                setTitle(currentFilePath.getAbsolutePath());
                if (currentFilePath.isDirectory()) {
                    listPane.getList().clearSelection();
                    listPane.loadDirectory(currentFilePath);
                }
            }
        });

        listPane.addMouseTracker(new MouseTracker());

        //Add the scroll panes to a split pane
        splitView = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        splitView.setOneTouchExpandable(true);
        splitView.setBottomComponent(viewerPane);
        splitView.setTopComponent(listPane);
        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT);
        splitPane.setLeftComponent(treePane);
        splitPane.setRightComponent(splitView);
        Dimension minimumSize = new Dimension(100, 50);
        listPane.setMinimumSize(minimumSize);
        treePane.setMinimumSize(minimumSize);
        splitPane.setDividerLocation(150);
        splitPane.setPreferredSize(new Dimension(500, 300));

        //Add the split pane to this frame
        setLocation( offset * openFrameCount, offset * openFrameCount);
        getContentPane().add(splitPane);
        openFrameCount++;
    }

    public void setContextMenu(JPopupMenu pm) {
      contextMenu = pm;
    }

    public JSVoyagerList getListPane() {
      return listPane;
    }

    public File getCurrentPath() {
      return currentFilePath;
    }

    public File getFile() {
      JSVoyagerFileSelection fs = listPane.getSelection();
      return fs.getSelectedFile(0);
    }

    public void createFolder(String folder) {
        File newDir = new File(currentFilePath, folder);
        newDir.mkdir();
        refreshAll();
    }

    public void upToParent() {
      if (currentFilePath.getParent() != null) {
        File newpath = new File(currentFilePath.getParent());
        if (newpath != null  && newpath.isDirectory()) {
          // change tree selection
          int count = currentTreePath.getPathCount();
          Object[] newTreePath = new Object[count - 1];
          for (int i = 0; i < count - 1; i++) {
            newTreePath[i] = currentTreePath.getPathComponent(i);
          }
          TreePath treePath = new TreePath(newTreePath);
          treePane.getTree().makeVisible(treePath);
          treePane.getTree().scrollPathToVisible(treePath);
          treePane.getTree().setSelectionPath(treePath);
        }
      }
    }

    public void refresh() {
        if (currentFilePath.isDirectory()) {
            listPane.getList().clearSelection();
            listPane.loadDirectory(currentFilePath);
	}
    }

    public void refreshAll() {
        if (currentFilePath.isDirectory()) {
            listPane.getList().clearSelection();
            listPane.loadDirectory(currentFilePath);
        }
        treePane.reloadSubTree(currentTreePath);
    }

    public void selectAll() {
        listPane.selectAll();
        refresh();
    }

    public void viewProperties() {
        File file = getFile();
        if (file != null) {
            JSVPropertiesDialog propDlg = new JSVPropertiesDialog(
                                             JOptionPane.getRootFrame(), 
                                             file);
            propDlg.show();
        }
        else
            JOptionPane.showMessageDialog(JOptionPane.getRootFrame(),
                          "No file is selected !");
    }

    public boolean deleteFiles() {
	JSVoyagerFileSelection selection = listPane.getSelection();
        JSVProgressDialog progressDlg = new JSVProgressDialog(
                                                     JOptionPane.getRootFrame(), 
                                                     " ... ", 0, 
                                                     selection.getSelectionSize());
        progressStop = false;
        DeletionThread dt = new DeletionThread(selection, progressDlg);
        dt.setPriority(Thread.MIN_PRIORITY);
        dt.start();
	return true;
    }

    class DeletionThread extends Thread {
      JSVoyagerFileSelection selection;     
      JSVProgressDialog progressDlg;
      boolean hasAllDeleted;

      public DeletionThread(JSVoyagerFileSelection selection, JSVProgressDialog progressDlg)
      {
          super();
	  this.selection = selection;
	  this.progressDlg = progressDlg;
      }

      public void run() {
        synchronized(operationLock) {
          hasAllDeleted = true;
          File[] selectedFiles = selection.getSelectedFiles();
          progressDlg.show();
          for (int i = 0; i < selectedFiles.length && hasAllDeleted; i++) {
            hasAllDeleted &= deleteFile(selectedFiles[i]);
            progressDlg.setProgress(i);
          }
          progressDlg.dispose();
          selection.clearSelection();
          refreshAll();
          operationLock.notify();
        }
      }

      public boolean deleteFile(File path) {
	boolean hasAllDeleted = true;
        if (path.isDirectory()) {
            JProgressBar progress = progressDlg.getProgressBar();
            int minValue = progress.getMinimum();
            int maxValue = progress.getMaximum();
            int Value = progress.getValue();
            String title = progressDlg.getTitle();
            String[] files = path.list();
            if (files != null) {
                progressDlg.setTitle(path.getAbsolutePath());
                progressDlg.setProgressRange(0, files.length);
                for (int i = 0; i < files.length  && hasAllDeleted; i++) {
                    File file = new File(path, files[i]);
                    hasAllDeleted &= deleteFile(file);
                    progressDlg.setProgress(i);
                    synchronized(progressLock) {
                      if (progressStop == true)
                        hasAllDeleted = false;
                    }
                }
            }
            try {
                hasAllDeleted &= path.delete();
                //System.out.println(hasAllDeleted + " than file has deleted : " + path.getName());
            } catch (SecurityException e) {
                System.out.println("Security violation : " + e.getLocalizedMessage());
            } catch (Exception e) {
                System.out.println("Deleting error : " + e);
            }
            progressDlg.setTitle(title);
            progressDlg.setProgressRange(minValue, maxValue);
            progressDlg.setProgress(Value);
        }
        else {
          try {
            hasAllDeleted &= path.delete();
            //System.out.println(hasAllDeleted + " than file has deleted : " + path.getName());
          } catch (SecurityException e) {
            System.out.println("Security violation : " + e.getLocalizedMessage());
          } catch (Exception e) {
            System.out.println("Deleting error : " + e);
          }
        }
	return hasAllDeleted;
      }
    }

    public void cutFiles() {
	filesClipboard = new JSVoyagerFileSelection(listPane.getSelection());
        moveFlag = true;
    }

    public void copyFiles() {
	filesClipboard = new JSVoyagerFileSelection(listPane.getSelection());
        moveFlag = false;
    }

    public boolean pasteFiles() {
	return copyingFiles(filesClipboard, currentFilePath, moveFlag);
    }

    public boolean copyingFiles(JSVoyagerFileSelection selection, File targetPath, boolean move) {
        JSVProgressDialog progressDlg = new JSVProgressDialog(
                                                     JOptionPane.getRootFrame(), 
                                                     " ... ", 0, 
                                                     selection.getSelectionSize());
        progressStop = false;
        CopyingThread ct = new CopyingThread(selection, targetPath, move, progressDlg);
        ct.setPriority(Thread.MIN_PRIORITY);
        ct.start();
	return true;
    }

    class CopyingThread extends Thread {
      JSVoyagerFileSelection selection;     
      JSVProgressDialog progressDlg;
      File targetPath;
      boolean hasAllCopied;
      boolean move;
      boolean overwriteAll;

      public CopyingThread(JSVoyagerFileSelection selection, File newPath, boolean move,
                           JSVProgressDialog progressDlg)
      {
          super();
	  this.selection = selection;
	  this.progressDlg = progressDlg;
          this.move = move;
          overwriteAll = false;
          targetPath = newPath;
      }

      public void run() {
        synchronized(operationLock) {
          hasAllCopied = true;
          File[] selectedFiles = selection.getSelectedFiles();
          progressDlg.show();
          for (int i = 0; i < selectedFiles.length && hasAllCopied; i++) {
            hasAllCopied &= cpFile(selectedFiles[i], targetPath);
            progressDlg.setProgress(i);
          }
          progressDlg.dispose();
          selection.clearSelection();
          refreshAll();
          operationLock.notify();
        }
      }

      public boolean cpFile(File path, File targetPath) {
	boolean hasAllCopied = true;
        try {
          if (path.isDirectory()) {
            File newTargetPath = new File(targetPath, path.getName());
            newTargetPath.mkdir();
            JProgressBar progress = progressDlg.getProgressBar();
            int minValue = progress.getMinimum();
            int maxValue = progress.getMaximum();
            int Value = progress.getValue();
            String title = progressDlg.getTitle();
            String[] files = path.list();
            if (files != null) {
                progressDlg.setTitle(path.getAbsolutePath());
                progressDlg.setProgressRange(0, files.length);
                for (int i = 0; i < files.length  && hasAllCopied; i++) {
                    File file = new File(path, files[i]);
                    hasAllCopied &= cpFile(file, newTargetPath);
                    progressDlg.setProgress(i);
                    synchronized(progressLock) {
                      if (progressStop == true)
                        hasAllCopied = false;
                    }
                }
                //System.out.println(hasAllCopied + " than directory has copied : " + path.getName());
            }
            else {
                  hasAllCopied &= copyOrMove(path, targetPath, move);
                  //System.out.println(hasAllCopied + " than file has copied : " + path.getName());
            }
            if (move && hasAllCopied)
                hasAllCopied &= path.delete();
            progressDlg.setTitle(title);
            progressDlg.setProgressRange(minValue, maxValue);
            progressDlg.setProgress(Value);
          }
          else {
              hasAllCopied &= copyOrMove(path, targetPath, move);
              //System.out.println(hasAllCopied + " than file has copied : " + path.getName());
          }
        } catch (SecurityException e) {
            System.out.println("Security violation : " + e.getLocalizedMessage());
        }
	return hasAllCopied;
      }

      boolean copyOrMove(File file, File target, boolean move) 
              throws SecurityException {
        File newFile;
        if (target.isDirectory()) {
          if (move) {
            newFile = new File(target, file.getName());
            return file.renameTo(newFile);
          }
          else {
            newFile = new File(target, file.getName());
            return copyFile(newFile, file);
          }
        }
        else { // isFile
          if (move) {
            return file.renameTo(target);
          }
          else {
            return copyFile(target, file);
          }
        }
      }

      boolean copyFile(File target, File file) throws SecurityException {
        Object[] options = { "No", "Yes", "All", "Cancel" };
        boolean overwrite = true;
        try {
          if (target.exists() && (overwriteAll == false)) {
            int retVal = JOptionPane.showOptionDialog(JOptionPane.getRootFrame(), 
                    "File " + target.getName() + " already exist ! Overwrite ?",
                    "Confirm ovewriting", JOptionPane.DEFAULT_OPTION, 
                    JOptionPane.QUESTION_MESSAGE,
                    null, options, options[0]);
            if ((retVal == JOptionPane.CLOSED_OPTION) || (retVal == 3)) // "Cancel"
                return false;
            else if (retVal == 2) // "All"
                overwriteAll = true;
            else if (retVal == 1) // "Yes"
                overwrite = true;
            else if (retVal == 0) // "No"
                overwrite = false;
          }
/*
            if (target.canWrite()) {
              if (file.canRead()) {
*/
            if (overwrite) {
                FileInputStream ifs = new FileInputStream(file);
                FileOutputStream ofs = new FileOutputStream(target);
                byte[] buffer = new byte[1024];
                int count;
                while ((count = ifs.read(buffer)) != -1)
                  ofs.write(buffer, 0, count);
                ifs.close();
                ofs.close();
/*
              }
              else {
                System.out.println("Can't write the file : " + target.getName());
                return false;
              }
            } 
            else {
              System.out.println("Can't read the file : " + file.getName());
              return false;
            }
*/
            }
        }
        catch(FileNotFoundException e) { 
            System.out.println("File Not Found : " + e);
            return false; 
        }
        catch(IOException e) { 
            System.out.println("I/O Error : " + e);
            return false; 
        }
        return true;
      }
    }

  public class JSVProgressDialog extends JDialog {

    JProgressBar progress;
    JLabel title;

    public JSVProgressDialog(Frame f, String title, int minValue, int maxValue) {
      super(f, "Progress", false);
      JPanel container = new JPanel();
      container.setLayout( new BorderLayout() );
      JPanel barPanel = new JPanel();
      barPanel.setLayout( new GridLayout(2, 1) );
      this.title = new JLabel(title);
      this.title.setAlignmentX(CENTER_ALIGNMENT);
      barPanel.add( this.title );
      progress = new JProgressBar();
      progress.setAlignmentX(CENTER_ALIGNMENT);
      progress.setMinimum(minValue);
      progress.setMaximum(maxValue);
      progress.setValue(0);
      progress.getAccessibleContext().setAccessibleName("Deleting Status.");
      barPanel.add( progress );
      JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout ( new FlowLayout(FlowLayout.CENTER) );
      JButton cancel = new JButton("Cancel");
      cancel.addActionListener(new ActionListener() {
                      public void actionPerformed(ActionEvent e) {
                           CancelPressed();
                        }});
      buttonPanel.add( cancel );
      getRootPane().setDefaultButton( cancel );
      container.add(barPanel, BorderLayout.CENTER);
      container.add(buttonPanel, BorderLayout.SOUTH);
      getContentPane().add(container);
      pack();
      centerDialog();
    }

    protected void centerDialog() {
      Dimension screenSize = this.getToolkit().getScreenSize();
      Dimension size = this.getSize();
      screenSize.height = screenSize.height/2;
      screenSize.width = screenSize.width/2;
      size.height = size.height/2;
      size.width = size.width/2;
      int y = screenSize.height - size.height;
      int x = screenSize.width - size.width;
      this.setLocation(x,y);
    }

    public void CancelPressed() {
      synchronized(progressLock) {
        progressStop = true;
        progressLock.notify();
      }
      this.setVisible(false);
    }

    public String getTitle() {
      return title.getText();
    }

    public void setTitle(String newtitle) {
      title.setText(newtitle);
    }

    public JProgressBar getProgressBar() {
      return progress;
    }

    public void setProgressRange(int minValue, int maxValue) {
      progress.setMinimum(minValue);
      progress.setMaximum(maxValue);
      progress.setValue(0);
    }

    public void setProgress(int i) {
      progress.setValue(i);
    }
  }

  public class JSVPropertiesDialog extends JDialog {
    JLabel size;

    public JSVPropertiesDialog(Frame f, File file) {
      super(f, "Properties", false);
      JPanel container = new JPanel();
      container.setLayout( new BorderLayout() );
      JPanel barPanel = new JPanel();
      barPanel.setLayout( new GridLayout(4, 1) );
      JLabel title = new JLabel(
                        ((file.isDirectory()) ? "Directory : " : "File : ")
                        + file.getName());
      title.setAlignmentX(CENTER_ALIGNMENT);
      barPanel.add( title );
      title = new JLabel("Location : " + file.getAbsolutePath());
      title.setAlignmentX(CENTER_ALIGNMENT);
      barPanel.add( title );
      size = new JLabel("Size : " + file.length());
      size.setAlignmentX(CENTER_ALIGNMENT);
      barPanel.add( size );
      Date d = new Date(file.lastModified());
      title = new JLabel("Last Modified : " + d.toString());
      title.setAlignmentX(CENTER_ALIGNMENT);
      barPanel.add( title );
/*
      JPanel permissionPanel = new JPanel();
      permissionPanel.setLayout( new GridLayout(permissions.length + 1, 1) );
      permissionPanel.add( new Label("Granted Permissions : ") );
      SecurityManager security = System.getSecurityManager();
      for (int i = 0; i < permissions.length; i++ ) {
        try {
          FilePermission perm = new FilePermission(file.getAbsolutePath(),
                                permissions[i]);
          System.out.println("permission : " + permissions[i] + " is being checked !");
          if (security != null)
              security.checkPermission(perm);
          else
              AccessController.checkPermission(perm);
          permissionPanel.add( new JLabel(permissions[i]));
          System.out.println(permissions[i] + "permission granted !");
        } catch (AccessControlException e) {
          System.out.println("AccessControlException" + e);
        }
      }
*/
      JPanel buttonPanel = new JPanel();
      buttonPanel.setLayout ( new FlowLayout(FlowLayout.CENTER) );
      JButton ok = new JButton("OK");
      ok.addActionListener(new ActionListener() {
                      public void actionPerformed(ActionEvent e) {
                          OKPressed();
                        }});
      buttonPanel.add( ok );
      getRootPane().setDefaultButton( ok );
      container.add(barPanel, BorderLayout.CENTER);
//      container.add(permissionPanel, BorderLayout.CENTER);
      container.add(buttonPanel, BorderLayout.SOUTH);
      getContentPane().add(container);
      pack();
      centerDialog();
    }

    protected void centerDialog() {
      Dimension screenSize = this.getToolkit().getScreenSize();
      Dimension size = this.getSize();
      screenSize.height = screenSize.height/2;
      screenSize.width = screenSize.width/2;
      size.height = size.height/2;
      size.width = size.width/2;
      int y = screenSize.height - size.height;
      int x = screenSize.width - size.width;
      this.setLocation(x,y);
    }

    public String getTotalSize() {
      return size.getText();
    }

    public void setTotalSize(String size) {
      this.size.setText(size);
    }

    public void OKPressed() {
      this.setVisible(false);
    }
  }

    class MouseTracker extends MouseAdapter 
    {
        public void mouseClicked(MouseEvent e) {
            if (e.getClickCount() == 2) {
                int index = listPane.getList().locationToIndex(e.getPoint());
                String fileName = listPane.getElementAt(index);
                File newpath = new File(currentFilePath, fileName);
                if (newpath.isDirectory()) {
                // change tree selection
                    JSVoyagerTreeNode newNode = treePane.findNode((JSVoyagerTreeNode)
                          currentTreePath.getLastPathComponent(), newpath);
                    if (newNode != null) {
                        int count = currentTreePath.getPathCount();
                        Object[] newTreePath = new Object[count + 1];
                        for (int i = 0; i < count; i++) {
                          newTreePath[i] = currentTreePath.getPathComponent(i);
                        }
                        newTreePath[count] = newNode;
                        TreePath treePath = new TreePath(newTreePath);
                        treePane.getTree().makeVisible(treePath);
                        treePane.getTree().scrollPathToVisible(treePath);
                        treePane.getTree().setSelectionPath(treePath);
                    }
                }
                else // show selected file
                    showAppropriateViewer(newpath);
            }
            else {
//                if (e.isPopupTrigger()) { 
                if ((e.getModifiers() & InputEvent.BUTTON3_MASK) == 
                                                InputEvent.BUTTON3_MASK) { 
                  if (contextMenu != null) {
                    contextMenu.show(e.getComponent(), e.getX(), e.getY());
                    refresh();
                  }
                }
            }
        }
    }

    public void showAppropriateViewer(File file)
    {
      try {
        String s = file.getAbsolutePath();
        s = "file:" + s;
        URL url = new URL(s);
        URLConnection conn = url.openConnection();
        String type = conn.getContentType();
        if (type.equals("application/zip")) {
            splitView.setBottomComponent(new JSVoyagerZipPanel(file,
                                                  foldersIcon, filesIcon));
        }
        else {
            viewerPane.setContent(file);
            splitView.setBottomComponent(viewerPane);
        }
      } catch (MalformedURLException e) {
          System.out.println("Malformed URL: " + e);
      } catch (IOException e) {
          System.out.println("IOException: " + e);
      }	
    }
}
