/*
 * @(#)JSVoyagerTree.java	0.1 1999/01/06
 * 
 * Copyright (c) 1999 IS Corp. All Rights Reserved.
 * @author Sergey Izvozchikov
 * 
 */

import com.sun.java.swing.JPanel;
import com.sun.java.swing.JTree;
import com.sun.java.swing.tree.DefaultMutableTreeNode;
import com.sun.java.swing.tree.DefaultTreeModel;
import com.sun.java.swing.event.TreeSelectionListener;
import com.sun.java.swing.event.TreeExpansionListener;
import com.sun.java.swing.event.TreeSelectionEvent;
import com.sun.java.swing.event.TreeExpansionEvent;
import com.sun.java.swing.tree.TreeSelectionModel;
import com.sun.java.swing.tree.TreePath;
//import com.sun.java.swing.tree.DefaultTreeCellRenderer;
import com.sun.java.swing.plaf.basic.BasicTreeCellRenderer;
import com.sun.java.swing.ImageIcon;
import java.util.Vector;
import java.io.IOException;
import java.io.File;
import java.util.Enumeration;
import com.sun.java.swing.JScrollPane;
import java.awt.*;
import java.awt.event.*;

public class JSVoyagerTree extends JPanel implements DnDTargetListener,
                                                     MouseMotionListener, 
													 MouseListener
{
    Vector mouseListeners = new Vector();
    Vector mouseMotionListeners = new Vector();

    DefaultTreeModel treeModel;
    JTree tree;
    TreePath currentPath, selectedPath;
    JSVoyagerTreeNode root; 
    JScrollPane treeView; 
    JSVoyagerPanel parent;

    public JSVoyagerTree(JSVoyagerPanel parent, File path) {

        // Create DropTarget to accept Dnd operations
        DnDEngine dndEngine = DnDEngine.getSystemDnDEngine();
        dndEngine.registerTarget(this);

        this.parent = parent;
        //Create the nodes.
        root = new JSVoyagerTreeNode("/", path);
        createNodes(root, path, 1);
        treeModel = new DefaultTreeModel(root);

        //Create a tree that allows one selection at a time.
        //tree = new JTree(root);
        tree = new JTree(treeModel);
        tree.getSelectionModel().setSelectionMode
                (TreeSelectionModel.SINGLE_TREE_SELECTION);

        tree.addMouseListener(this);
        tree.addMouseMotionListener(this);
        //Listen for when the selection changes. 
        tree.addTreeSelectionListener(new TreeSelectionListener() { 
            public void valueChanged(TreeSelectionEvent e) { 
                JSVoyagerTreeNode node = (JSVoyagerTreeNode) 
                                   (e.getPath().getLastPathComponent()); 
                //if (! node.isChildrenLoaded()) { 
                    setWaitCursor();
                    createNodes(node, node.getFile(), 1);
                    setDefaultCursor();
                //} 
            } 
        }); 

        tree.addTreeExpansionListener(new TreeExpansionListener() { 
            public void treeExpanded(TreeExpansionEvent e) { 
                JSVoyagerTreeNode node = (JSVoyagerTreeNode) 
                                   (e.getPath().getLastPathComponent()); 
                //if (! node.isChildrenLoaded()) { 
                    setWaitCursor();
                    createNodes(node, node.getFile(), 1);
                    setDefaultCursor();
                //} 
            } 
            public void treeCollapsed(TreeExpansionEvent e) { 
            } 
        }); 
 
 
        /*
         * Set the icon for leaf nodes.
         * Note: In the Swing 1.0.x release, we used
         * swing.plaf.basic.BasicTreeCellRenderer.
         */
//        DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
/*
        BasicTreeCellRenderer renderer = new BasicTreeCellRenderer();
        renderer.setLeafIcon(new ImageIcon("images/folder.gif"));
        renderer.setOpenIcon(new ImageIcon("images/fopened.gif"));
        renderer.setClosedIcon(new ImageIcon("images/folder.gif"));
        tree.setCellRenderer(renderer);
*/

        //Create the scroll pane and add the tree to it. 
        treeView = new JScrollPane(tree);

        treeView.setMinimumSize(new Dimension(100, 50));

        //Add the split pane to panel
        setLayout(new BorderLayout());
        add(treeView, BorderLayout.CENTER);
    }

    public JTree getTree() {
        return tree;
    }

    public JSVoyagerTreeNode getRoot() {
	return root;
    }

    public void setWaitCursor() {
        tree.setCursor(new Cursor(Cursor.WAIT_CURSOR));
    }

    public void setDefaultCursor() {
        tree.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
    }

    public JSVoyagerTreeNode findNode(JSVoyagerTreeNode parent, File newpath) {
        for (Enumeration nodes = parent.children() ; nodes.hasMoreElements() ;) {
            JSVoyagerTreeNode node = (JSVoyagerTreeNode) nodes.nextElement();
//            if (node.getFile().getName().equals(newpath.getName()))
            if (node.getFile().equals(newpath))
		return node;
        }
	return null;
    }

    public void reloadSubTree(TreePath treePath) {
        JSVoyagerTreeNode node = (JSVoyagerTreeNode)
                                           treePath.getLastPathComponent();
        tree.collapsePath(treePath);
        node.removeChildren();
        //node.removeAllChildren();
        treeModel.reload(node);
        node.setChildrenLoaded(false);
        setWaitCursor();
        createNodes(node, node.getFile(), 1);
        setDefaultCursor();
        treeModel.reload(node);
        tree.setSelectionPath(treePath);
        //tree.expandPath(treePath);
/*
        node.removeChildren();
        node.setChildrenLoaded(false);
        setWaitCursor();
        createNodes(node, node.getFile(), 1);
        setDefaultCursor();
        tree.setSelectionPath(treePath);
        tree.collapsePath(treePath);
*/
    }

    // Add listeners for when the selection changes. 
    public void addTreeSelectionListener(TreeSelectionListener listener) { 
        tree.addTreeSelectionListener(listener);
    }

    // Remove listeners for when the selection changes. 
    public void removeTreeSelectionListener(TreeSelectionListener listener) { 
        tree.removeTreeSelectionListener(listener);
    }

    private void createNodes(JSVoyagerTreeNode root, File path, int lvl) {
      JSVoyagerTreeNode newnode = null;
      File file; 
      String[] files = null; 
 
      if (! root.isChildrenLoaded()) { 
        files = path.list(); 
	if (files != null) {
          for (int i = 0; i < files.length; i++) { 
            file = new File(path, files[i]); 
            if (file.isDirectory()) { 
              newnode = new JSVoyagerTreeNode(file.getName(), file); 
              //if (lvl <= 2)
              //    createNodes(newnode, file, lvl + 1);
              root.add(newnode);
            } 
          } 
	}
        root.setChildrenLoaded(true);
      }
/*
      else {
        for (int i = 0; i < root.getChildCount(); i++) { 
            newnode = (JSVoyagerTreeNode) root.getChildAt(i);
            if (lvl <= 2)
                createNodes(newnode, newnode.getFile(), lvl + 2);
        }
      }
*/
    }

    // DnDTargetListener implementation
    public void dragOver(DnDDragEvent e) {
        Point p = e.getLocation();
        Point transl = treeView.getViewport().getViewPosition();
        p.translate(transl.x, transl.y);
        selectedPath = tree.getPathForLocation(p.x, p.y);
        if (selectedPath != null) {
//            tree.setSelectionPath(selectedPath);
            e.acceptDrop();
        }
        else
            e.rejectDrop();
    }

    public boolean dropAccept(DnDDropEvent e) {
        Point p = new Point(e.getLocation());
        Point transl = treeView.getViewport().getViewPosition();
        p.translate(transl.x, transl.y);
        selectedPath = tree.getPathForLocation(p.x, p.y);
        JSVoyagerTreeNode node = (JSVoyagerTreeNode)
                                   (selectedPath.getLastPathComponent());
        if (selectedPath != null) {
//            tree.setSelectionPath(selectedPath);
            System.out.println(" Accept drop to : " + node.getFile().getName());
            e.acceptDrop();
            return true;
        }
        else {
            e.rejectDrop();
            return false;
        }
    }

    public void drop(DnDDropEvent dtde) {
        System.out.println("JSVoyagerTree: DnD Object has been droped !");
        int action = dtde.getDropAction(); 
        if (action == DnDEngine.ACTION_COPY ||
            action == DnDEngine.ACTION_MOVE) {
            System.out.println("JSVoyagerTree: Do the DnD operation !");
            Point p = new Point(dtde.getLocation());
            Point transl = treeView.getViewport().getViewPosition();
            p.translate(transl.x, transl.y);
            selectedPath = tree.getPathForLocation(p.x, p.y);
            if (selectedPath != null) {
                boolean move = false;
                dtde.acceptDrop();
                System.out.print("JSVoyagerTree: Drop acepted to : ");
                JSVoyagerFileSelection filesSelected = (JSVoyagerFileSelection)
                                                       dtde.getTransferable();
                JSVoyagerTreeNode node = (JSVoyagerTreeNode)
                                   (selectedPath.getLastPathComponent());
                System.out.println(node.getFile().getAbsolutePath());
                if (action == DnDEngine.ACTION_COPY)
                    move = false;
                else if (action == DnDEngine.ACTION_MOVE) 
                    move = true;
                System.out.println("JSVoyagerTree: " + move +  " to move files !");
                parent.copyingFiles(filesSelected, node.getFile(), move);
            }
            else
                dtde.rejectDrop();
        }
        else
            dtde.rejectDrop();
    }

    public synchronized void addMouseListener(MouseListener l) {
      mouseListeners.addElement(l);
    }

    public synchronized void removeMouseListener(MouseListener l) {
      mouseListeners.removeElement(l);
    }

    public synchronized void addMouseMotionListener(MouseMotionListener l) {
      mouseMotionListeners.addElement(l);
    }

    public synchronized void removeMouseMotionListener(MouseMotionListener l) {
      mouseMotionListeners.removeElement(l);
    }

    // MouseListener implementation
    public void mouseClicked(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseListener target = (MouseListener) targets.elementAt(i);
        target.mouseClicked(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

    public void mousePressed(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseListener target = (MouseListener) targets.elementAt(i);
        target.mousePressed(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

    public void mouseReleased(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseListener target = (MouseListener) targets.elementAt(i);
        target.mouseReleased(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

    public void mouseEntered(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseListener target = (MouseListener) targets.elementAt(i);
        target.mouseEntered(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

    public void mouseExited(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseListener target = (MouseListener) targets.elementAt(i);
        target.mouseExited(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

    // MouseMotionListener implementation
    public void mouseMoved(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseMotionListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseMotionListener target = (MouseMotionListener) targets.elementAt(i);
        target.mouseMoved(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

    public void mouseDragged(MouseEvent e) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) mouseMotionListeners.clone();
      }
      for (int i = 0; i < targets.size(); i++) {
        MouseMotionListener target = (MouseMotionListener) targets.elementAt(i);
        target.mouseDragged(new MouseEvent(this, e.getID(), e.getWhen(),
                            e.getModifiers(), e.getX(), e.getY(), 
                            e.getClickCount(), e.isPopupTrigger()));
      }
    }

}
