/*
 * @(#)JSVoyagerZipPanel.java	0.1 1999/01/06
 * 
 * Copyright (c) 1999 IS Corp. All Rights Reserved.
 * @author Sergey Izvozchikov
 * 
 */

import com.sun.java.swing.*;
import com.sun.java.swing.JOptionPane;
import com.sun.java.swing.event.*;
import com.sun.java.swing.table.*;
//import com.sun.java.swing.DefaultListCellRenderer;

import java.util.Date;
import java.util.*;
import java.util.zip.*;
import java.io.IOException;
import java.io.File;
import java.awt.*;
import java.awt.event.*;
//import java.awt.dnd.*;

public class JSVoyagerZipPanel extends JPanel /* implements DragSourceListener,
                                                     DragGestureListener */
{

    public static final int LIST_MODE = 1;
    public static final int TABLE_MODE = 2;

    public static ImageIcon foldersIcon;
    public static ImageIcon filesIcon;

//    DragSource dndSource;
//    DragGestureRecognizer dndGuesture;

    JSVoyagerFileSelection filesSelection;
    ZipFile currentZipFile;

    JList list; 
    JTable table;
    CardLayout layout;
    JSVoyagerZipPanelModel dataModel;
    String newline = "\n";
    String nullString = ""; 
    String folderString = "Directory"; 
    ListSelectionModel listSelectionModel;
    int viewMode;

    public JSVoyagerZipPanel(File file, ImageIcon foldersIcon, ImageIcon filesIcon)
    {
        this.foldersIcon = foldersIcon;
        this.filesIcon = filesIcon;

        //The data 
        String[] columnNames = { "Name", "Size", "Compressed size", "Modification Date" };
        // Types of the columns.
        Class[] columnClasses = { String.class, String.class, String.class, Date.class};

        // Transferable object with information about selected files
        filesSelection = new JSVoyagerFileSelection();

        //Build the model.
        dataModel = new JSVoyagerZipPanelModel(columnNames, columnClasses);
        try {
            currentZipFile = new ZipFile(file);
        }
        catch (Exception e) {
            System.out.println("File open error : " + e);
        }
        dataModel.setZipFile(currentZipFile);

        list = new JList(dataModel);
        list.setCellRenderer(new DefaultListCellRenderer() {
            public Component getListCellRendererComponent(JList l, 
                                                          Object value,
                                                          int i,
                                                          boolean s,
                                                          boolean f) {
                ZipEntry e = currentZipFile.getEntry((String) value);
                if (e.isDirectory())
                  setIcon(JSVoyagerZipPanel.foldersIcon);
                else
                  setIcon(JSVoyagerZipPanel.filesIcon);
                return super.getListCellRendererComponent(l,
                                                          value,
                                                          i, s, f);
            }
        });

        listSelectionModel = list.getSelectionModel();
        listSelectionModel.addListSelectionListener(
                                new SharedListSelectionHandler());
        listSelectionModel.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        JScrollPane listPane = new JScrollPane(list,
                               JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                               JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

        table = new JTable(dataModel);
        table.setSelectionModel(listSelectionModel);
        table.setShowGrid(false);
        table.getTableHeader().setReorderingAllowed(true);
        JScrollPane tablePane = new JScrollPane(table);

/*
                if (newMode.equals("SINGLE_SELECTION")) {
                    listSelectionModel.setSelectionMode(
                        ListSelectionModel.SINGLE_SELECTION);
                } else if (newMode.equals("SINGLE_INTERVAL_SELECTION")) {
                    listSelectionModel.setSelectionMode(
                        ListSelectionModel.SINGLE_INTERVAL_SELECTION);
                } else {
                    listSelectionModel.setSelectionMode(
                        ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
                }
*/

/*
        // Initialize DnD Peer
        try {
          dndSource = DragSource.getDefaultDragSource();
          dndGuesture = dndSource.createDragGestureRecognizer( 
                 Class.forName( "java.awt.dnd.MouseDragGestureRecognizer"),
                               list, DnDConstants.ACTION_COPY | 
                               DnDConstants.ACTION_MOVE, this);
          if (dndGuesture == null) 
            throw new InvalidDnDOperationException();
        }
        catch (ClassNotFoundException e) {
            System.out.println("DragGestureRecognizer creation error : " + e);
        }
        catch (InvalidDnDOperationException ex) {
            System.out.println("DragGestureRecognizer creation error : " + ex);
        }
*/

        //do the layout
        layout = new CardLayout();
        setLayout(layout);
        add(listPane, "list");
        add(tablePane, "table");
        listPane.setPreferredSize(new Dimension(300, 100));
        tablePane.setPreferredSize(new Dimension(300, 100));
        layout.show(this, "list");
        viewMode = LIST_MODE;
    }

    public JList getList() {
        return list;
    }

    public JSVoyagerFileSelection getSelection() {
        return filesSelection;
    }

    public String getElementAt(int index) {
        return (String) dataModel.getValueAt(index, 0);
    }

    public void selectAll() {
        ListModel lm = list.getModel();
        //listSelectionModel.setSelectionInterval(0, lm.getSize() - 1);
        list.setSelectionInterval(0, lm.getSize() - 1);
    }

    public void addMouseTracker(MouseListener l) {
        list.addMouseListener(l);
        table.addMouseListener(l);
    }

    public void setWaitCursor() {
        if (viewMode == LIST_MODE) {
            list.setCursor(new Cursor(Cursor.WAIT_CURSOR));
        } 
        else if (viewMode == TABLE_MODE) {
            table.setCursor(new Cursor(Cursor.WAIT_CURSOR));
        }
    }

    public void setDefaultCursor() {
        if (viewMode == LIST_MODE) {
            list.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        } 
        else if (viewMode == TABLE_MODE) {
            table.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
        }
    }

    public void showList() {
        layout.show(this, "list");
        viewMode = LIST_MODE;
    }

    public void showTable() {
        layout.show(this, "table");
        viewMode = TABLE_MODE;
    }

    public void loadZipFile(File file) {
        setWaitCursor();
        if (file != null) {
          try {
            currentZipFile = new ZipFile(file);
          }
          catch (Exception e) {
            System.out.println("File open error : " + e);
          }
          dataModel.setZipFile(currentZipFile);
          filesSelection.clearSelection();
          list.revalidate();
          table.revalidate();
          list.repaint();
          table.repaint();
          repaint();
        }
        setDefaultCursor();
    }
 
/*
    protected Frame getFrame() {
        for (Container p = getParent(); p != null; p = p.getParent()) {
            if (p instanceof Frame) {
                return (Frame) p;
            }
        }
        return null;
    }
*/

/*
    public void dragGestureRecognized(DragGestureEvent dge) {
        startDnDOperation( dge, dge.getComponent(), dge.getDragOrigin());
    }

    public void startDnDOperation(DragGestureEvent e, Component source,
                                  Point startPoint) {
          try {
            dndSource.startDrag( e,
                      DragSource.DefaultCopyNoDrop, filesIcon.getImage(),
                      startPoint, filesSelection, this);
          } catch (InvalidDnDOperationException ex) {
            System.out.println("DnD Operation can't be started ! " + ex);
          } catch (Exception ex) {
            System.out.println("Error ! " + ex);
          }
    }

    // DragSourceListener implementation
    public void dropActionChanged(DragSourceDragEvent dsde) {
//        if ((InputEvent.BUTTON1_MASK & dsde.getGestureModifiers()) == 0)
//            dsde.getDragSourceContext().cancelDrag();
    }

    public void dragEnter(DragSourceDragEvent dsde) {
        int dropAction = dsde.getDropAction();
        DragSourceContext dsc = dsde.getDragSourceContext();
        if ( dropAction == DnDConstants.ACTION_COPY ) {
            dsc.setCursor(DragSource.DefaultCopyDrop);
        } else if ( dropAction == DnDConstants.ACTION_MOVE ) {
            dsc.setCursor(DragSource.DefaultMoveDrop);
        }
        else {
            dsc.setCursor(DragSource.DefaultCopyNoDrop);
        }
    }

    public void dragOver(DragSourceDragEvent dsde) {
        int dropAction = dsde.getDropAction();
        DragSourceContext dsc = dsde.getDragSourceContext();
        if ( dropAction == DnDConstants.ACTION_COPY ) {
            dsc.setCursor(DragSource.DefaultCopyDrop);
        } else if ( dropAction == DnDConstants.ACTION_MOVE ) {
            dsc.setCursor(DragSource.DefaultMoveDrop);
        }
        else {
            dsc.setCursor(DragSource.DefaultCopyNoDrop);
        }
    }

    public void dragExit(DragSourceEvent dse) { }

    public void dragDropEnd(DragSourceDropEvent dsde) {
      try {
        if ( dsde.getDropSuccess() ) {
            if ( dsde.getDropAction() == DnDConstants.ACTION_COPY ) { }
			else if ( dsde.getDropAction() == DnDConstants.ACTION_MOVE ) { }
        }
        else {
        }
      } catch (Exception ex) {
          System.out.println("Error ! " + ex);
      }
    }
*/

    class SharedListSelectionHandler implements ListSelectionListener {
        public void valueChanged(ListSelectionEvent e) { 
            ListSelectionModel lsm = (ListSelectionModel) e.getSource();
            if (! lsm.isSelectionEmpty()) {
		filesSelection.clearSelection();
                // Find out which indexes are selected.
                int minIndex = lsm.getMinSelectionIndex();
                int maxIndex = lsm.getMaxSelectionIndex();
                for (int i = minIndex; i <= maxIndex; i++) {
                    if (lsm.isSelectedIndex(i)) {
//                        filesSelection.addFile(new File(currentZipFile, getElementAt(i)));
                    }
                }
            }
        }
    }

    class JSVoyagerZipPanelModel extends AbstractListModel
                          implements TableModel {

        public String[] columnNames;
        public Class[] columnClasses;
        public String[] files;
        ZipFile zipFile;

        public JSVoyagerZipPanelModel(String[] columnNames,
                                  Class[] columnClasses) {
            super();
            this.columnNames = columnNames;
            this.columnClasses = columnClasses;
            files = null;
        }

        public void setZipFile(ZipFile zipFile) {
          //if (zipFile != null) {
            this.zipFile = zipFile;
            //int zipSize = zipFile.size();
            Vector children = new Vector();
            for (Enumeration e = zipFile.entries();
                     e.hasMoreElements(); ) {
                ZipEntry ze = (ZipEntry) e.nextElement();
                children.addElement(ze);
            }
            if (children.size() > 0) {
                files = new String[children.size()];
                for (int i = 0; i < children.size(); i++) {
                    ZipEntry ze = (ZipEntry) children.elementAt(i);
                    files[i] = ze.getName();
                }
                super.fireContentsChanged(this, 0, files.length - 1);
            }
          //}
        }

        private TableModel tableModel = new AbstractTableModel() {
            public String getColumnName(int column) {
                return columnNames[column];
            }
/*
            public Class getColumnClass(int column) {
                return columnClasses[column];
            }
*/
            public int getRowCount() { 
                if (files != null)
                  return files.length;
                else
                  return 0;
            }
            public int getColumnCount() {
                return columnNames.length;
            }
            public Object getValueAt(int row, int column) {
              if (files != null) {
                ZipEntry file = zipFile.getEntry(files[row]); 
                if (file.isDirectory()) { 
                  switch(column) {
                    case 0: return file.getName();
                    case 1: return folderString;
                    case 2: return nullString;
                    case 3: return nullString;
                  }
                } else {
                  switch(column) {
                    case 0: return file.getName();
                    case 1: Long len = new Long(file.getSize()); return len.toString();
                    case 2: Long clen = new Long(file.getCompressedSize()); return clen.toString();
                    case 3: Date d = new Date(file.getTime()); return d.toString();
                  }
                }
              }
              return null;
            }
            public boolean isCellEditable(int row, int column) {
//                if (column == 0) return true;
                return false;
            }
            public void setValueAt(Object value, int row, int column) {
/*
                String newValue = (String)value;
                String[] rowData = (String [])elementAt(row);
                rowData[column] = newValue;
*/
                list.revalidate(); 
            }
        };

        public int getSize() {
            if (files != null) 
                return files.length;
            else
                return 0;
        }

        public Object getElementAt(int i) {
            if (files != null)
                return files[i];
            else
                return null;
        }

        //Implement the TableModel interface.
        public int getRowCount() {
            return tableModel.getRowCount();
        }
        public int getColumnCount() {
            return tableModel.getColumnCount();
        }
        public String getColumnName(int columnIndex) {
            return tableModel.getColumnName(columnIndex);
        }
        public Class getColumnClass(int columnIndex) {
            return tableModel.getColumnClass(columnIndex);
        }
        public boolean isCellEditable(int rowIndex, int columnIndex) {
            return tableModel.isCellEditable(rowIndex, columnIndex);
        }
        public Object getValueAt(int rowIndex, int columnIndex) {
            return tableModel.getValueAt(rowIndex, columnIndex);
        }
        public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
            tableModel.setValueAt(aValue, rowIndex, columnIndex);
        }
        public void addTableModelListener(TableModelListener l) {
            tableModel.addTableModelListener(l);
        }
        public void removeTableModelListener(TableModelListener l) {
            tableModel.removeTableModelListener(l);
        }

    }

class DefaultListCellRenderer extends JLabel implements ListCellRenderer {
    public DefaultListCellRenderer() {
        setOpaque(true);
    }

    public Component getListCellRendererComponent(
        JList list, 
        Object value, 
        int index, 
        boolean isSelected, 
        boolean cellHasFocus) 
    {
        setText((String) value);
        setBackground(isSelected ? Color.red : Color.white);
        setForeground(isSelected ? Color.white : Color.black);
        return this;
    }
}
}
